/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */

module;
#include <cassert>

export module DefaultFraction;

import std;	

namespace DefaultFraction
{
	export class Fraction
	{
	public:
		Fraction(int n, int d) :n_{n}, d_{d}
		{
			assert(!(n < 0) && !(d <= 0));

			int frac_gcd = std::gcd(n, d);
			n_ /= frac_gcd;
			d_ /= frac_gcd;

			if (n_ == d_)
			{
				n_ = 1;
				d_ = 1;
			}
		}
		//bool operator == (const Fraction& rhs) const = default;
		auto operator <=>(const Fraction& rhs) const = default;
		friend std::ostream& operator<<(std::ostream& os, const Fraction& p);

		int numer() const
		{
			return n_;
		}

		int denom() const
		{
			return d_;
		}

	private:
		int n_, d_;
	};

	export std::ostream& operator<<(std::ostream& os, const Fraction& f)
	{
		return os << f.n_ << "/" << f.d_ << " ";
	}
}