/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */

module;
#include <cassert>

export module Fraction;

import std;	

export class Fraction
{
public:
	Fraction(int n, int d) :n_{n}, d_{d}
	{
		// Assume numerator non-negative and numerator strictly positive
		assert(!(n < 0) && !(d <= 0));

		// Simplify at construction:
		int frac_gcd = std::gcd(n, d);
		n_ /= frac_gcd;
		d_ /= frac_gcd;

		if (n_ == d_)
		{
			n_ = 1;
			d_ = 1;
		}
	}

	// Old way:
	//bool operator ==(const Fraction& rhs) const;
	//bool operator !=(const Fraction& rhs) const;
	//bool operator <(const Fraction& rhs) const;
	//bool operator >(const Fraction& rhs) const;
	//bool operator <=(const Fraction& rhs) const;
	//bool operator >=(const Fraction& rhs) const;

	bool operator ==(const Fraction& rhs) const = default;

	std::strong_ordering operator <=>(const Fraction& rhs) const
	{
		if (n_ * rhs.d_ < rhs.n_ * d_)
		{
			return std::strong_ordering::less;
		}
		else if (*this == rhs)		// `==` defined with `default` in header	
		//else if (n_ == rhs.n_ && d_ == rhs.d_)	// Could remove the == operator and use this instead
		{
			return std::strong_ordering::equivalent;
		}
		else
		{
			return std::strong_ordering::greater;
		}
	}

	friend std::ostream& operator<<(std::ostream& os, const Fraction& p);

	int numer() const
	{
		return n_;
	}

	int denom() const
	{
		return d_;
	}


private:
	int n_, d_;
};

export std::ostream& operator<<(std::ostream& os, const Fraction& f)
{
	return os << f.n_ << "/" << f.d_ << " ";
}